@@ -22,6 +22,7 @@ Used to restrict which commands can be run via trusted SSH keys."""
 
 
 from . import rsync as rsync_checker
+from . import scp as scp_checker
 
 import argparse
 import glob
@@ -30,6 +31,7 @@ import ipaddress
 import os
 import pprint
 import re
+import shutil
 import subprocess
 import sys
 import textwrap
@@ -119,6 +121,10 @@ class AuthProgs(object):  # pylint: disable-msg=R0902
         else:
             self.logfh = False
 
+        # Stores result of shutils.which
+        # key is program name, val is realpath.
+        self.which_cache = {}
+
         if kwargs.get('authprogs_binary'):
             self.authprogs_binary = kwargs['authprogs_binary']
         else:
@@ -145,6 +151,7 @@ class AuthProgs(object):  # pylint: disable-msg=R0902
 
         self.subvalidators = {}
         self.subvalidators['rsync'] = rsync_checker.RsyncValidator(self)
+        self.subvalidators['scp'] = scp_checker.ScpValidator(self)
 
     def __del__(self):
         if self.logfh:
@@ -229,7 +236,7 @@ class AuthProgs(object):  # pylint: disable-msg=R0902
         If realpath is set then convert to real path after resolving symlinks
         Always calls abspath to create a path that starts at / and resolve ..
 
-        Perform glob expansion of name and returns list of canonicalized
+        Perform glob expansion of name and returns list of canonicalised
         file names
         """
         path = self._os_path_abspath(path)
@@ -247,7 +254,7 @@ class AuthProgs(object):  # pylint: disable-msg=R0902
     def realpath(self, path):
         """Return the real path for the given file path.
 
-        Returns canonicalized file name (eliminate symlinks, handle '..')
+        Returns canonicalised file name (eliminate symlinks, handle '..')
         """
         return self._os_path_realpath(path)
 
@@ -419,56 +426,44 @@ class AuthProgs(object):  # pylint: disable-msg=R0902
             self.original_command_list[:], rule
         )
 
-    def find_match_scp(self, rule):  # pylint: disable-msg=R0911,R0912
-        """Handle scp commands."""
+    def _whichbin(self, name):
+        """shutil.which wrapper for unit test mocking."""
+        return shutil.which(name)
 
-        orig_list = []
-        orig_list.extend(self.original_command_list)
-        binary = orig_list.pop(0)
-        allowed_binaries = ['scp', '/usr/bin/scp']
-        if binary not in allowed_binaries:
-            self.logdebug(
-                'skipping scp processing - binary "{}" '
-                'not in approved list.\n'.format(binary)
-            )
+    def valid_binary(self, name, allowed):
+        """Determine if name is a valid binary in allowed.
+
+        Does realpath expansion and returns actual pathname.
+        """
+        # Return from cache if set and allowed
+        if name in self.which_cache:
+            if self.which_cache[name] in allowed:
+                return self.which_cache[name]
+            else:
+                return
+        else:
+            self.which_cache[name] = None
+
+        # Find it in our path
+        foundbin = self._whichbin(name)
+
+        if not foundbin:
             return
 
-        filepath = orig_list.pop()
-        arguments = orig_list
+        # Convert to realpath, store, and return
+        foundbin = os.path.realpath(foundbin)
+        if foundbin not in allowed:
+            return
 
-        if '-f' in arguments:
-            if not rule.get('allow_download'):
-                self.logdebug('scp denied - downloading forbidden.\n')
-                return
+        # Store this successful name in cache
+        self.which_cache[name] = foundbin
+        return foundbin
 
-        if '-t' in arguments:
-            if not rule.get('allow_upload'):
-                self.log('scp denied - uploading forbidden.\n')
-                return
-
-        if '-r' in arguments:
-            if not rule.get('allow_recursion'):
-                self.log('scp denied - recursive transfers forbidden.\n')
-                return
-
-        if '-p' in arguments:
-            if not rule.get('allow_permissions', 'true'):
-                self.log('scp denied - set/getting permissions forbidden.\n')
-                return
-
-        if rule.get('files'):
-            files = rule.get('files')
-            if not isinstance(files, list):
-                files = [files]
-            if filepath not in files:
-                self.log(
-                    'scp denied - file "{}" - not in approved '
-                    'list {}\n'.format(filepath, files)
-                )
-                return
-
-        # Allow it!
-        return {'command': self.original_command_list}
+    def find_match_scp(self, rule):  # pylint: disable-msg=R0911,R0912
+        """Handle scp commands."""
+        return self.subvalidators['scp'].validate_command(
+            self.original_command_list[:], rule
+        )
 
     def find_match_command(self, rule):
         """Return a matching (possibly munged) command, if found in rule."""
@@ -568,7 +563,7 @@ class AuthProgs(object):  # pylint: disable-msg=R0902
         command_info = {
             'from': self.get_client_ip(),
             'keyname': self.keyname,
-            'ssh_original_comand': self.original_command_string,
+            'ssh_original_command': self.original_command_string,
             'time': time.time(),
         }
 
